#ifndef MountieHistory_h
#define MountieHistory_h

#include <QString>
#include <QList>

// Used to record info about previously referenced mountpoints as well as
// how often they have been referenced (list is kept sorted by most
// frequently used mountpoint names).
//
class MountieHistory {
public:

   MountieHistory();

   MountieHistory(QString init_mountpoint, int init_use_count);

   MountieHistory(const MountieHistory & cpy);

   ~MountieHistory();

   MountieHistory &
   operator =(const MountieHistory & cpy);

   bool
   operator <(const  MountieHistory & cmp) const;

   QString mountpoint;

   int use_count;
};

typedef QList<MountieHistory> MountieHistoryList;

#endif // MountieHistory_h
