#include <QtGui>
#include "MountieDialog.h"
#include "MountieMemory.h"

int
main(int argc, char ** argv) {
   QApplication app(argc, argv);
   QIcon icon(":/new/prefix1/dudley-50x50.png");
   app.setWindowIcon(icon);
   MountieDialog * dig = new MountieDialog;
   QSize size = MountieMemory::Get()->GetSize();
   dig->resize(size);
   dig->show();
   return app.exec();
}
