#ifndef PrognameDialog_h
#define PrognameDialog_h

#include <QtGui>
#include "ui_mount_unmount.h"

class PrognameDialog : public QDialog {
   Q_OBJECT

public:

   PrognameDialog(QWidget * parent = 0);

   ~PrognameDialog();

private slots:

   void
   BrowseButtonPushed();

   void
   ResetButtonPushed();

   void
   OKButtonPushed();

   void
   CancelButtonPushed();

   void
   Accepted();

private:

   Ui::Progname ui;
};

#endif // PrognameDialog_h
