#include "MountieHistory.h"

MountieHistory::MountieHistory() : use_count(0) {
   return;
}

MountieHistory::MountieHistory(QString init_mountpoint, int init_use_count)
 : mountpoint(init_mountpoint), use_count(init_use_count) {
   return;
}

MountieHistory::MountieHistory(const MountieHistory & cpy)
 : mountpoint(cpy.mountpoint),
   use_count(cpy.use_count) {
   return;
}

MountieHistory::~MountieHistory() {
   return;
}

MountieHistory &
MountieHistory::operator =(const MountieHistory & cpy) {
   mountpoint = cpy.mountpoint;
   use_count = cpy.use_count;
   return *this;
}

bool
MountieHistory::operator <(const MountieHistory & cmp) const {
   if (use_count == cmp.use_count) {
      return mountpoint < cmp.mountpoint;
   } else if (use_count > cmp.use_count) {
      return true;
   } else {
      return false;
   }
}
